<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.pedigreeonline.com
 * @since      1.0.0
 *
 * @package    Pedigree_Online
 * @subpackage Pedigree_Online/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Pedigree_Online
 * @subpackage Pedigree_Online/admin
 * @author     PedigreeOnline.com, Inc <info@pedigreeonline.com>
 */
class Pedigree_Online_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	* Quick reference to the slug we use for the admin page
	*/
	private $page_slug;

	/**
	* Quick reference to our settings option_name
	*/
	private $settings_name;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->page_slug = trim(Pedigree_Online::OPTION_PREFIX,'_');
		$this->settings_name = Pedigree_Online::OPTION_PREFIX . 'settings';

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pedigree_Online_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pedigree_Online_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

    wp_enqueue_style( 'wp-color-picker');

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/backend.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Pedigree_Online_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Pedigree_Online_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

    wp_enqueue_script( 'wp-color-picker');

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/backend.js', array( 'jquery' ), $this->version, false );

	}

	/**
	* Sets up the options menu for the backend
	*
	* @since 1.0.0
	*/
	public function define_menu() {

    //add_options_page( __('Pedigree Online Configuation'), __('Pedigree Online'), 'manage_options', $this->page_slug, array( $this, 'render_page' ) );
    add_menu_page( __('Pedigree Online Configuation'), __('Pedigree Online'), 'manage_options', $this->page_slug, array( $this, 'render_page' ), 'dashicons-share', 79 );

	}

  /**
  * Actually provides HTML from a partial for the settings / options page - defined in MENU section
  *
  * @since 1.0.0
  */
	public function render_page() {

	  require plugin_dir_path( __FILE__ ) . 'partials/pedigree-online-admin-display.php';

	}



	/**
	* Sets up the setting parameters using the settings API for use on the settings page
	*
	* @since 1.0.0
	*/
	public function define_settings() {

    // If we're not on the page we want to be on (as definied in the menu setup or options.php for POST) then leave...
    if ( empty( $GLOBALS['pagenow'] ) || ! in_array( $GLOBALS['pagenow'], array('admin.php','options.php') ) ) {
      return;
    }

	  // Make sure the options exist in the database
	  add_option( $this->settings_name );
	  // This is deleted via uninstall.php

    // A setting belongs to a group - can be general, reading, etc... or something custom
    // This allows wordpress to call-back a sanitze function after POST before it updates the setting
    register_setting(
      $this->page_slug, // Grouping name, this affects things like validation - Needs to match used in settings_fields();
      $this->settings_name, // The actual option_key name
      array($this, 'validate_data')
    );

    /** API Section **/
    add_settings_section(
        'api', // Section name to create
        __( 'API Configuration'), // Title
        array($this, 'render_page_section'), // Callback for output
        $this->page_slug // Which admin page this loads on
    );

      add_settings_field(
          Pedigree_Online::OPTION_PREFIX . 'api_token', // ID of the field
          __( 'API Token'),  // Label
          array($this, 'render_input_field'), // Callback for field HTML
          $this->page_slug, // Which page this applies to
          'api', // Which section this applies to
          array(
            'id' => 'api_token',
            'type' => 'text',
            'size' => 40,
            'class' => 'large-text',
            'placeholder' => 'EG: oafdsF6dsfakvmwphd74jcDkw730Dkg82',
            'helptext' => 'Enter your API token here. For more information on acquiring an API token, visit <a href="https://www.pedigreeonline.com/wordpress">PedigreeOnline.com/wordpress</a>.',
          )
      );

    /** Colors Section **/
    add_settings_section(
        'colors', // Section name to create
        __( 'Customize the Report'), // Title
        array($this, 'render_page_section'), // Callback for output
        $this->page_slug // Which admin page this loads on
    );

      add_settings_field(
          Pedigree_Online::OPTION_PREFIX . 'use_custom_colors', // ID of the field
          __( 'Apply Custom Colors?'),  // Label
          array($this, 'render_input_field'), // Callback for field HTML
          $this->page_slug, // Which page this applies to
          'colors', // Which section this applies to
          array(
            'id' => 'use_custom_colors',
            'type' => 'checkbox',
            //'helptext' => 'Check this box if you want to enable your custom colors to be applied to reports on your Wordpress site.',
          )
      );

      add_settings_field(
          Pedigree_Online::OPTION_PREFIX . 'male_background_color', // ID of the field
          __( 'Male Background Color'),  // Label
          array($this, 'render_input_field'), // Callback for field HTML
          $this->page_slug, // Which page this applies to
          'colors', // Which section this applies to
          array(
            'id' => 'male_background_color',
            'type' => 'text',
            'size' => 10,
            'placeholder' => '#RRGGBB',
            'class' => 'color-picker',
            //'helptext' => 'Enter a valid #HEX or named CSS color. Leave blank for default color.',
          )
      );

      add_settings_field(
          Pedigree_Online::OPTION_PREFIX . 'male_text_color', // ID of the field
          __( 'Male Name Color'),  // Label
          array($this, 'render_input_field'), // Callback for field HTML
          $this->page_slug, // Which page this applies to
          'colors', // Which section this applies to
          array(
            'id' => 'male_text_color',
            'type' => 'text',
            'size' => 10,
            'placeholder' => '#RRGGBB',
            'class' => 'color-picker',
            //'helptext' => 'Enter a valid #HEX or named CSS color. Leave blank for default color.',
          )
      );

      add_settings_field(
          Pedigree_Online::OPTION_PREFIX . 'female_background_color', // ID of the field
          __( 'Female Background Color'),  // Label
          array($this, 'render_input_field'), // Callback for field HTML
          $this->page_slug, // Which page this applies to
          'colors', // Which section this applies to
          array(
            'id' => 'female_background_color',
            'type' => 'text',
            'size' => 10,
            'placeholder' => '#RRGGBB',
            'class' => 'color-picker',
            //'helptext' => 'Enter a valid #HEX or named CSS color. Leave blank for default color.',
          )
      );

      add_settings_field(
          Pedigree_Online::OPTION_PREFIX . 'female_text_color', // ID of the field
          __( 'Female Name Color'),  // Label
          array($this, 'render_input_field'), // Callback for field HTML
          $this->page_slug, // Which page this applies to
          'colors', // Which section this applies to
          array(
            'id' => 'female_text_color',
            'type' => 'text',
            'size' => 10,
            'placeholder' => '#RRGGBB',
            'class' => 'color-picker',
            //'helptext' => 'Enter a valid #HEX or named CSS color. Leave blank for default color.',
          )
      );

      add_settings_field(
          Pedigree_Online::OPTION_PREFIX . 'custom_css', // ID of the field
          __( 'Custom CSS'),  // Label
          array($this, 'render_input_field'), // Callback for field HTML
          $this->page_slug, // Which page this applies to
          'colors', // Which section this applies to
          array(
            'id' => 'custom_css',
            'type' => 'textarea',
            'size' => 20,
            'placeholder' => ".pedigreeonline table td.pedigree-gender-male {\n  background: blue;\n}\n\n.pedigreeonline table td.pedigree-gender-female {\n  background: pink;\n}\n",
            'helptext' => 'Enter your own custom CSS here. The report is inside a <code>'.esc_html('<div class="pedigreeonline">').'</code> wrapper, so start all your CSS definitions with <code>.pedigreeonline ...</code>',
          )
      );

	}

	/**
	* In charge of rendering the settings section called 'api'
	*
	* @since 1.0.0
	*/
	public function render_page_section( $args ) {

	    if ( file_exists( $path = plugin_dir_path( __FILE__ ) . 'partials/pedigree-online-admin-section-'. $args['id'] .'.php' ) ) {

	      require $path;

	    }

  }

  /** Now the individual callbacks for each of the field parts **/
  public function render_input_field( $args ) {

    $options = get_option( $this->settings_name );

    if ( file_exists( $path = plugin_dir_path( __FILE__ ) . 'partials/pedigree-online-admin-input-'. $args['id'] .'.php' ) ) {

	      require $path;

	  }

    require plugin_dir_path( __FILE__ ) . 'partials/pedigree-online-admin-input-field.php';

  }


  /**
  * Validates the incoming setting data as an array - let's do some sanity checks and return the final array
  *
  * @since 1.0.0
  */
  public function validate_data($data) {

    // Start with the API token
    if ( $data['api_token'] && ( strlen( $data['api_token'] = sanitize_text_field( $data['api_token'] ) )  != 64 || ! preg_match('/^[a-zA-Z0-9]+$/', $data['api_token']) ) ) {

      add_settings_error( $this->settings_name, 'api_token_wrong_length', __('The API token is the wrong length. It should be a 64 character string consisting of letters and numbers.') );

    }

    // Check on the colors.
    foreach ( array('male','female') as $sex ) {

      foreach ( array('background','text') as $type ) {

        if ( $data[$sex . '_' . $type . '_color'] && ! ( $data[$sex . '_' . $type . '_color'] = sanitize_hex_color( $data[$sex . '_' . $type . '_color'] ) ) ) {

          // Throw warning
          add_settings_error( $this->settings_name, 'invalid_color', __('The color for '. $sex . ' '. $type . ' is not in proper format properly entered.', 'warning') );

        }

      }

    }

    return $data;

  }

  /***
  * An ajax function to call clear_trasients()
  *
  * @since 1.0.0
  */
  public function ajax_clear_cache() {

    Pedigree_Online_Functions::clear_transients();

    wp_send_json_success(array('message' => __('Report Data cache has been cleared.')));

  }



}
